# -*- coding: utf-8 -*-
"""A Django password hasher that leverages Postgres pgcrypto encryption."""

from django.db import connections
from django.contrib.auth.hashers import (BasePasswordHasher, mask_hash)
from django.utils.datastructures import SortedDict
from django.utils.crypto import constant_time_compare
from django.utils.translation import ugettext_noop as _

from .settings import PGCRYPTOAUTH_DATABASE


class PgCryptoPasswordHasher(BasePasswordHasher):
    """
    Password hasher which is intended to provide password continuity from the
    legacy pgcrypto-encoded password (not recommended)

    pgcrypto$pghash

    Configured to use the crypt() method from the Postgres pgcrypto extension.
    NOTE: Postgres and pgcrypto are required.
    """
    algorithm = 'pgcrypto'

    def salt(self):
        """
        Generates a salt via pgcrypto.gen_salt('md5').
        """
        cursor = connections[PGCRYPTOAUTH_DATABASE].cursor()
        cursor.execute("SELECT gen_salt('md5')")
        return cursor.fetchall()[0][0]

    def encode(self, password, salt=None):
        assert password
        if not salt:
            salt = self.salt()

        cursor = connections[PGCRYPTOAUTH_DATABASE].cursor()
        cursor.execute("SELECT crypt(%s, %s)", [password, salt])
        pghash = cursor.fetchall()[0][0]
        return "%s$%s" % (self.algorithm, pghash)

    def verify(self, password, encoded):
        algorithm, pghash = encoded.split('$', 1)
        assert algorithm == self.algorithm
        encoded_2 = self.encode(password, pghash)
        return constant_time_compare(encoded, encoded_2)

    def safe_summary(self, encoded):
        algorithm, pghash = encoded.split('$', 1)
        assert algorithm == self.algorithm
        return SortedDict([
            (_('algorithm'), algorithm),
            (_('hash'), mask_hash(pghash)),
        ])



# -*- coding: utf-8 -*-
"""Test suite for django-pgcryptoauth."""

from django.contrib.auth import authenticate
from django.contrib.auth.models import User
from django.db import connection, transaction
from django.test import TestCase

from pgcryptoauth.hashers import PgCryptoPasswordHasher


class PgCryptoAuthTests(TestCase):
    """Test case for django-pgcryptoauth."""
    hasher = PgCryptoPasswordHasher()
    username = u'kingbuzzo'
    password = u'sY0CVj0L56'
    pghash = u'$1$3yGxidZQ$q1ntm5wERWVI2Xq4UbRx8.'
    encoded = u'pgcrypto$%s' % pghash

    def setUp(self):
        cursor = connection.cursor()
        cursor.execute("CREATE EXTENSION pgcrypto")
        transaction.commit_unless_managed()
        self.user = User.objects.create_user(username=self.username)
        self.user.password = self.encoded
        self.user.save()

    def tearDown(self):
        self.user.delete()

    def test_encode(self):
        """
        Test encoder with good password.
        """
        encoded = self.hasher.encode(self.password, self.pghash)
        self.assertEqual(encoded, self.encoded)

    def test_encode_nosalt(self):
        """
        Test encoder without a salt.
        """
        encoded = self.hasher.encode(self.password)
        self.assertTrue(self.hasher.verify(self.password, encoded))

    def test_bad_encode(self):
        """
        Test encoder with bad password.
        """
        encoded = self.hasher.encode('bad password', self.pghash)
        self.assertNotEqual(encoded, self.encoded)

    def test_verify(self):
        """
        Test verifier.
        """
        self.assertTrue(self.hasher.verify(self.password, self.encoded))

    def test_authenticate(self):
        """
        Test authentication.
        """
        user = authenticate(username=self.username, password=self.password)
        self.assertTrue(isinstance(user, User))

    def test_bad_authenticate(self):
        """
        Test authentication with bad password.
        """
        user = authenticate(username=self.username, password='bad password')
        self.assertFalse(isinstance(user, User))

    def test_summary(self):
        """
        Test summary string generation.
        """
        summary = self.hasher.safe_summary(self.encoded)
        self.assertIn('algorithm', summary)
        self.assertIn('hash', summary)
        self.assertEqual(summary['algorithm'], 'pgcrypto')
        expected = self.pghash[0:6] + ('*' * (len(self.pghash)-6))
        self.assertEqual(summary['hash'], expected)



from setuptools import setup, find_packages
import os

import pgcryptoauth


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

README = read('README.rst')

setup(
    name='django-pgcryptoauth',
    version=pgcryptoauth.__version__,
    description='Django hasher for pgcrypto encoded passwords.',
    long_description=README,
    url='https://github.com/tomatohater/django-pgcryptoauth',
    author='Drew Engelson',
    author_email='drew@engelson.net',
    license='MIT',
    zip_safe=False,
    packages=find_packages(),
    include_package_data=True,
    package_data={},
    classifiers = [
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)



# -*- coding: utf-8 -*-
"""Default settings for package (overridable with Django settings)."""

from django.conf import settings

#
# Specify the name of the database connection to use. Default to 'default'
#
PGCRYPTOAUTH_DATABASE = getattr(settings, 'PGCRYPTOAUTH_DATABASE', 'default')
